@@ -1,9 +1,11 @@
 package File::Append::TempFile;
 
-use 5.006;
+use v5.010;
 use strict;
 use warnings;
 
+use version; our $VERSION = version->declare("0.06");
+
 =head1 NAME
 
 File::Append::TempFile - Perl extension for appending data to files
@@ -38,12 +40,11 @@ file without affecting the original in any way.
 
 =cut
 
-use Fcntl;
+use File::Basename qw(basename dirname);
+use File::Temp;
 
 our @ISA = qw();
 
-our $VERSION = '0.05';
-
 our $debug = 0;
 
 my %tempfiles;
@@ -67,8 +68,12 @@ sub new
 	my $class = ref $proto || $proto;
 	my $self;
 
-	$self = { 'fname' => undef, 'tmpfname' => undef, 'f' => undef,
-	    'err' => undef, 'debug' => undef };
+	$self = {
+		fname => undef,
+		f => undef,
+		err => undef,
+		debug => undef
+	};
 	bless $self, $class;
 	$tempfiles{$self} = $self;
 	return $self;
@@ -85,8 +90,8 @@ sub err($ $)
 {
 	my ($self, $err) = @_;
 
-	$self->{'err'} = $err if @_ > 1;
-	return $self->{'err'};
+	$self->{err} = $err if @_ > 1;
+	return $self->{err};
 }
 
 =item diag ([FLAG])
@@ -100,8 +105,8 @@ sub diag($ $)
 {
 	my ($self, $debug) = @_;
 
-	$self->{'debug'} = $debug if @_ > 1;
-	return $self->{'debug'};
+	$self->{debug} = $debug if @_ > 1;
+	return $self->{debug};
 }
 
 =item begin_work (FILENAME)
@@ -117,31 +122,31 @@ method.
 sub begin_work($ $)
 {
 	my ($self, $fname) = @_;
-	my ($tmpfname, $orig, $f);
+	my ($orig, $f);
 	my @stat;
 
-	if ($self->{'f'}) {
+	if ($self->{f}) {
 		return undef unless $self->rollback();
 	}
-	$self->{'fname'} = $self->{'tmpfname'} = $self->{'f'} = undef;
+	$self->{fname} = $self->{f} = undef;
 
-	if (!open($orig, '<', $fname)) {
+	if (!open $orig, '<', $fname) {
 		$self->err("Opening $fname: $!");
 		return undef;
 	}
 	@stat = stat $orig;
-	$tmpfname = sprintf '%s.%05d.%04d', $fname, $$, int rand 10000;
-	if (!sysopen($f, $tmpfname, O_WRONLY | O_EXCL | O_CREAT, 0600)) {
-		$self->err("Creating temporary file $tmpfname: $!");
+	$f = File::Temp->new(basename($fname).'.XXXXXX',
+	    DIR => dirname($fname));
+	if (!defined $f) {
+		$self->err("Creating a temporary file for $fname: $!");
 		return undef;
 	}
 	return undef unless $self->do_copy($orig, $f);
 	close $orig;
 
-	$self->{'fname'} = $fname;
-	$self->{'tmpfname'} = $tmpfname;
-	$self->{'f'} = $f;
-	$self->{'stat'} = [ @stat ];
+	$self->{fname} = $fname;
+	$self->{f} = $f;
+	$self->{stat} = [ @stat ];
 	return 1;
 }
 
@@ -155,13 +160,13 @@ any way until C<commit()> is invoked.
 sub add_line($ $)
 {
 	my ($self, $line) = @_;
-	my $f = $self->{'f'};
+	my $f = $self->{f};
 
-	if (!defined($f)) {
+	if (!defined $f) {
 		$self->err("Cannot add_line() to an unopened tempfile");
 		return undef;
 	}
-	$self->debug("RDBG about to add a line to $self->{tmpfname} for $self->{fname}\n");
+	$self->debug("RDBG about to add a line to $f for $self->{fname}\n");
 	if (!(print $f $line)) {
 		$self->err("Could not add to the tempfile: $!");
 		return undef;
@@ -184,38 +189,38 @@ object should be C<begin_work()>.
 sub commit($)
 {
 	my ($self) = @_;
+	my $f = $self->{f};
 
-	if (!defined($self->{'f'}) || !defined($self->{'tmpfname'}) ||
-	    !defined($self->{'fname'})) {
+	if (!defined $f || !defined $self->{fname}) {
 		$self->err("Cannot commit an unopened tempfile");
 		return undef;
 	}
-	$self->debug("RDBG about to commit $self->{tmpfname} to $self->{fname}\n");
+	$self->debug("RDBG about to commit $f to $self->{fname}\n");
 
 	# Fix stuff up
-	if (defined($self->{'stat'})) {
+	if (defined $self->{stat}) {
 		# Mode
-		if (!chmod($self->{'stat'}->[2], $self->{'tmpfname'})) {
+		if (!chmod $self->{stat}->[2], $f) {
 			$self->err("Could not chmod $self->{stat}->[2] ".
-			    $self->{'tmpfname'}.": $!");
+			    "$f: $!");
 			return undef;
 		}
 		# Owner & group
-		if (!chown($self->{'stat'}->[4], $self->{'stat'}->[5],
-		    $self->{'tmpfname'})) {
+		if (!chown $self->{stat}->[4], $self->{stat}->[5], $f) {
 			$self->err("Could not chown $self->{stat}->[4], ".
-			    $self->{'stat'}->[5].", $self->{tmpfname}: $!");
+			    "$self->{stat}->[5], $f: $!");
 			return undef;
 		}
 	}
 	
-	if (!rename($self->{'tmpfname'}, $self->{'fname'})) {
-		$self->err("Renaming $self->{tmpfname} to $self->{fname}: $!");
+	if (!rename $f, $self->{fname}) {
+		$self->err("Renaming $f to $self->{fname}: $!");
 		return undef;
 	}
-	close $self->{'f'};
-	$self->debug("RDBG successfully committed $self->{tmpfname} to $self->{fname}\n");
-	$self->{'fname'} = $self->{'tmpfname'} = $self->{'f'} = undef;
+	$f->unlink_on_destroy(0);
+	close $f;
+	$self->debug("RDBG successfully committed $f to $self->{fname}\n");
+	$self->{fname} = $self->{f} = undef;
 	return 1;
 }
 
@@ -236,20 +241,14 @@ sub rollback($)
 	my ($self) = @_;
 
 	$self->debug(ref($self)."->rollback() for $self->{fname}\n");
-	if (defined($self->{'tmpfname'})) {
-		$self->debug("RDBG removing $self->{tmpfname}\n");
-		if (!unlink($self->{'tmpfname'})) {
-			$self->err("Removing $self->{tmpfname}: $!");
-			return undef;
-		}
-		undef $self->{'tmpname'};
+	if (defined $self->{f}) {
+		my $f = $self->{f};
+		$self->debug("RDBG closing and removing $f\n");
+		$f->unlink_on_destroy(1);
+		close $f;
+		undef $self->{f};
 	}
-	if (defined($self->{'f'})) {
-		$self->debug("RDBG closing the file\n");
-		close $self->{'f'};
-		undef $self->{'f'};
-	}
-	undef $self->{'fname'};
+	undef $self->{fname};
 	$self->debug("RDBG rollback seems complete\n");
 	return 1;
 }
@@ -272,7 +271,7 @@ sub debug($ $)
 {
 	my ($self, $msg) = @_;
 
-	if ($self->{'debug'} || $debug) {
+	if ($self->{debug} || $debug) {
 		print STDERR $msg;
 	}
 }
@@ -303,7 +302,7 @@ END
 	print STDERR "RDBG File::Append::TempFile END block\n" if $debug;
 	print STDERR "RDBG ".keys(%tempfiles)."\n" if $debug;
 	foreach (keys %tempfiles) {
-		$tempfiles{$_}->rollback() if $tempfiles{$_}->{'tmpfname'};
+		$tempfiles{$_}->rollback() if $tempfiles{$_}->{f};
 	}
 }
 
@@ -336,8 +335,6 @@ This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself, either Perl version 5.8.7 or,
 at your option, any later version of Perl 5 you may have available.
 
-$Ringlet: TempFile.pm 1635 2007-12-27 14:06:01Z roam $
-
 =cut
 
 1;
